package com.landray.kmss.sys.attachment.service.spring;

import org.springframework.beans.BeansException;
import java.util.Map;
import com.landray.kmss.framework.plugin.core.config.IExtension;
import com.landray.kmss.sys.attachment.interfaces.ISysAtt3rdDownloadService;
import com.landray.kmss.sys.attachment.interfaces.ISysAtt3rdDownloadUrl;
import com.landray.kmss.framework.service.plugin.Plugin;
import com.alibaba.fastjson.JSONObject;
import org.springframework.context.ApplicationEvent;
import com.landray.kmss.util.UserUtil;
import com.landray.kmss.util.StringUtil;
import java.util.Date;
import com.landray.kmss.sys.attachment.model.SysAtt3rdDownloadQueue;
import com.landray.kmss.common.model.IBaseModel;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.slf4j.Logger;
import com.landray.kmss.sys.attachment.event.SysAtt3rdDownloadQueueDownloadEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.context.ApplicationContextAware;
import com.landray.kmss.sys.attachment.service.ISysAtt3rdDownloadQueueService;
import com.landray.kmss.sys.metadata.interfaces.ExtendDataServiceImp;

public class SysAtt3rdDownloadQueueServiceImp extends ExtendDataServiceImp implements ISysAtt3rdDownloadQueueService, ApplicationContextAware, ApplicationListener<SysAtt3rdDownloadQueueDownloadEvent>
{
    private static final Logger logger;
    private ApplicationContext applicationContext;
    
    static {
        logger = LoggerFactory.getLogger(SysAtt3rdDownloadQueueServiceImp.class);
    }
    
    public String add(final IBaseModel modelObj) throws Exception {
        final SysAtt3rdDownloadQueue model = (SysAtt3rdDownloadQueue)modelObj;
        model.setFdStatus(Integer.valueOf(0));
        model.setFdCreateTime(new Date());
        if (StringUtil.isNull(model.getFdCreatedBy())) {
            model.setFdCreatedBy(UserUtil.getUser().getFdId());
        }
        final String add = super.add((IBaseModel)model);
        if (model.isFdSync()) {
            this.handleDownloadQueue(model);
        }
        else {
            this.applicationContext.publishEvent((ApplicationEvent)new SysAtt3rdDownloadQueueDownloadEvent(model));
        }
        return add;
    }
    
    public void onApplicationEvent(final SysAtt3rdDownloadQueueDownloadEvent event) {
        if (event == null || event.getSource() == null) {
            return;
        }
        final SysAtt3rdDownloadQueue queue = event.getSource();
        try {
            this.handleDownloadQueue(queue);
        }
        catch (final Exception e) {
            SysAtt3rdDownloadQueueServiceImp.logger.error("下载第三方文件执行异常，queue = {}", JSONObject.toJSONString((Object)queue), e);
        }
    }
    
    private void handleDownloadQueue(final SysAtt3rdDownloadQueue queue) throws Exception {
        final IExtension[] downloadUrlExts = Plugin.getExtensions("com.landray.kmss.sys.attachment.3rd.download.url", "*");
        final IExtension downloadUrlExt = Plugin.getExtension(downloadUrlExts, "key", String.valueOf(queue.getFdType()));
        if (downloadUrlExt == null) {
            SysAtt3rdDownloadQueueServiceImp.logger.error("扩展点[com.landray.kmss.sys.attachment.3rd.download.url] key = {} 没有对应的实现", queue.getFdType());
            return;
        }
        final ISysAtt3rdDownloadUrl urlService = (ISysAtt3rdDownloadUrl)Plugin.getParamValue(downloadUrlExt, "provider");
        final String url = urlService.generateUrl(queue);
        final IExtension[] downloadServiceExts = Plugin.getExtensions("com.landray.kmss.sys.attachment.3rd.download.service", "*");
        final IExtension downloadServiceExt = Plugin.getExtension(downloadServiceExts, "key", String.valueOf(queue.getFdDownloadMethod()));
        if (downloadServiceExt == null) {
            SysAtt3rdDownloadQueueServiceImp.logger.error("扩展点[com.landray.kmss.sys.attachment.3rd.download.service] key = {} 没有对应的实现", queue.getFdDownloadMethod());
            return;
        }
        final ISysAtt3rdDownloadService downloadService = (ISysAtt3rdDownloadService)Plugin.getParamValue(downloadServiceExt, "provider");
        final Map<String, String> data = downloadService.download(queue, url);
        queue.setFdUpdateTime(new Date());
        if (data != null && "success".equals(data.get("result"))) {
            queue.setFdStatus(Integer.valueOf(1));
        }
        else {
            queue.setFdStatus(Integer.valueOf(2));
        }
        this.update((IBaseModel)queue);
    }
    
    public void setApplicationContext(final ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }
}
